#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/dir.h>

#include "filer.h"
#include "pad.h"
#include "screen.h"

BROWSERFILEINFO files[MAX_ENTRY];
BROWSERFILEINFO *sortfiles[MAX_ENTRY];
int nfiles;

// extension structures
const struct {
	char *szExt;
	int nExtId;
} stExtentions[] = {
	{"gb",EXT_GB},
	{"gbc",EXT_GB},
	{"sgb",EXT_GB},
	{"gz",EXT_GZ},
	{"zip",EXT_ZIP},
	{"tch",EXT_TCH},
	{NULL, EXT_UNKNOWN}
};
unsigned long getExtId(const char *szFilePath)
{
	char *pszExt;
	
	pszExt = strrchr(szFilePath, '.');
	if (pszExt) {
		pszExt++;
		int i;
		for (i=0; stExtentions[i].nExtId!=EXT_UNKNOWN; i++){
			if (!stricmp(stExtentions[i].szExt,pszExt)){
				return stExtentions[i].nExtId;
			}
		}
	}
	
	return EXT_UNKNOWN;
}

////////////////////////////////////////////////////////////////////////
// AC add start
void SJISCopy(BROWSERFILEINFO *a, unsigned char *file)
{
	unsigned char ca;
	int i;
	int len=strlen(a->name);
	
	for(i=0;i<=len;i++){
		ca = a->name[i];
		if (((0x81 <= ca)&&(ca <= 0x9f))
		|| ((0xe0 <= ca)&&(ca <= 0xef))){
			file[i++] = ca;
			file[i] = a->name[i];
		}
		else{
			if(ca>='a' && ca<='z') ca-=0x20;
			file[i] = ca;
		}
	}

}
int fileCompare(BROWSERFILEINFO *a, BROWSERFILEINFO *b)
{
    unsigned char file1[0x108];
    unsigned char file2[0x108];
	unsigned char ca, cb;
	int i, n, ret;

	if(a->attr == b->attr){
		SJISCopy(a, file1);
		SJISCopy(b, file2);
		n=strlen((char*)file1);
		for(i=0; i<=n; i++){
			ca=file1[i]; cb=file2[i];
			ret = ca-cb;
			if(ret!=0) return ret;
		}
		return 0;
	}
	
	if(a->attr) return -1;
	else return 1;
}
// AC add end

void sort_files(BROWSERFILEINFO **a, int left, int right)
{
	BROWSERFILEINFO *tmp, *pivot;
	int i, p;
	
	if (left < right) {
		pivot = a[left];
		p = left;
		for (i=left+1; i<=right; i++) {
			if (fileCompare(a[i],pivot)<0){
				p=p+1;
				tmp=a[p];
				a[p]=a[i];
				a[i]=tmp;
			}
		}
		a[left] = a[p];
		a[p] = pivot;
		sort_files(a, left, p-1);
		sort_files(a, p+1, right);
	}
}
////////////////////////////////////////////////////////////////////////

void getDir(const char *path, unsigned long ext)
{
    DIR_ITER* dp;

	char filename[MAXPATHLEN];
	struct stat fstat;
	int root=1;
	
	nfiles = 0;
	
	dp = diropen( path );

	if (dp == NULL) {
		sprintf(path, "fat:/");
		dp = diropen(path);
		if (dp == NULL) return;
	}
	
    while( dirnext(dp, filename, &fstat) == 0 )
    {
        if ((getExtId(filename) & ext) || fstat.st_mode & S_IFDIR)
        {
            if (strcmp(filename, ".")==0)
                continue;
            if (strcmp(path, "/")==0 && strcmp(filename, "..")==0)
                continue;

            sprintf(files[nfiles].name, "%s", filename);
            sprintf(files[nfiles].path, "%s/%s", path, filename);
            files[nfiles].size   = fstat.st_size;
            files[nfiles].attr   = (fstat.st_mode & S_IFDIR) ? 1 : 0;
            sortfiles[nfiles] = &files[nfiles];
            ++nfiles;
        }
    }

    dirclose(dp);

	if(root)
		sort_files(sortfiles, 0, nfiles-1);
	else
		sort_files(sortfiles+1, 0, nfiles-2);

}

int getFilePath(char *fullpath, unsigned long ext)
{
	int sel=0, top=0, rows=20, x, y, i, up=0;
	char path[MAX_PATH], oldDir[MAX_NAME], *p;
	extern u32 *xfb[];
	extern long pg_drawframe, pg_showframe;
	
	strcpy(path, fullpath);
	
	p = strrchr(path, '/');
	if (p){
		*p = 0;
	}
	
	getDir(path, ext);

	for(;;){
		readpad();
		if(new_pad & PAD_BUTTON_A){
			if(sortfiles[sel]->attr){
				if(!strcmp(sortfiles[sel]->name,"..")){
					up=1;
				}else{
					strcat(path,"/");
					strcat(path,sortfiles[sel]->name);
					getDir(path, ext);
					sel=0;
				}
			}else{
				break;
			}
		}else if(new_pad & PAD_BUTTON_B){
			return 0;
		}else if(new_pad & PAD_BUTTON_Y){
			up=1;
		}else if(new_pad & PAD_BUTTON_UP){
			sel--;
		}else if(new_pad & PAD_BUTTON_DOWN){
			sel++;
		}else if((new_pad & PAD_BUTTON_LEFT) || (new_pad & PAD_TRIGGER_L)){
			sel-=rows/2;
		}else if((new_pad & PAD_BUTTON_RIGHT) || (new_pad & PAD_TRIGGER_R)){
			sel+=rows/2;
		}
		
		if(up){
			oldDir[0]=0;
			if(path[0]){
				p=(strrchr(path,'/'));
				strcpy(oldDir, p+1);
				*p = 0;
				getDir(path, ext);
				sel=0;
			}
			for(i=0; i<nfiles; i++) {
				if(!strcmp(oldDir,sortfiles[i]->name)) {
					sel=i;
					top=sel-3;
					break;
				}
			}
			up=0;
		}
		
		if(top > nfiles-rows)	top=nfiles-rows;
		if(top < 0)				top=0;
		if(sel >= nfiles)		sel=nfiles-1;
		if(sel < 0)				sel=0;
		if(sel >= top+rows)		top=sel-rows+1;
		if(sel < top)			top=sel;
		
		VIDEO_ClearFrameBuffer (vmode, xfb[pg_drawframe], COLOR_BLACK);
		
		x=28; y=30;
		for(i=0; i<rows; i++){
			if(top+i >= nfiles) break;
			if(sortfiles[top+i]->attr){
				mh_print(x, y, "/", (top+i==sel) ? COLOR_YELLOW : COLOR_WHITE);
				mh_print(x+10, y, sortfiles[top+i]->name, (top+i==sel)?COLOR_YELLOW:COLOR_WHITE);
			}
			else{
				mh_print(x, y, sortfiles[top+i]->name, (top+i==sel)?COLOR_YELLOW:COLOR_WHITE);
			}
			y+=20;
		}
		
		VIDEO_SetNextFramebuffer(xfb[pg_drawframe]);
		VIDEO_Flush();
		VIDEO_WaitVSync();
		pg_showframe ^= 1;
		pg_drawframe ^= 1;
	}
	
	strcpy(fullpath, path);
	strcat(fullpath, "/");
	strcat(fullpath, sortfiles[sel]->name);
	
	VIDEO_ClearFrameBuffer (vmode, xfb[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (vmode, xfb[1], COLOR_BLACK);
	
	return 1;
}
