#include <gccore.h>
#include <ogcsys.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fat.h>
#include <wiiuse/wpad.h>
#include <sys/stat.h>

#include "gb.h"
#include "sound.h"
#include "filer.h"
#include "screen.h"
#include "menu.h"
#include "whl_arg.h"


static inline unsigned long long mftb(void)
{
	register u32 tbu, tbl, tmp;

	asm volatile(
		"0:\n\t"
		"mftbu %0\n\t"
		"mftb %1\n\t"
		"mftbu %2\n\t"
		"cmpw %0, %2\n\t"
		"bne- 0b"
		: "=r"(tbu), "=r"(tbl), "=r"(tmp)
		:
		: "cc");

	return (((u64)tbu)<<32) + tbl;
}

unsigned long long getusec()
{
	return mftb() / 60.766322;
}

void mainloop(void)
{
	int line;
	int count=0;
	
	wavoutStart();
	
	for(;;) {
		for(line=0; line<154; line++)
			gb_run();
		
		if(tvmode == VI_PAL)
			count++;
		if (count < 5){
			VIDEO_SetNextFramebuffer(xfb[pg_drawframe]);
			VIDEO_Flush();
			VIDEO_WaitVSync();
			pg_showframe ^= 1;
			pg_drawframe ^= 1;
			skip = 0;
		}
		else{
			skip = 1;
			count = 0;
		}
		
		if(bMenu){
			wavoutStop();
			rin_menu();
			wavoutStart();
			bMenu = 0;
		}
	}
}

int main (int argc, char **argv)
{
	int RetVal = 0;
	char * Drivename, * Path;

	PAD_Init();
	WPAD_Init();
	screenInit();
	wavoutInit();
	configInit();

    fatInitDefault();
	bMenu = 1;

//////////////////////////////////////
// Command line patch by Hell Hibou //
//////////////////////////////////////

	if (argc > 1)
	{
		SplitDrivePath (argv[1], &Drivename, &Path);
		
		if (stricmp (Drivename, /*"frontsd"*/ "fat3") == 0) 
		{
			fatSetDefaultInterface (PI_INTERNAL_SD);
			mkdir ("/EMULATOR", S_IRWXU);
			mkdir ("/EMULATOR/RIN", S_IRWXU);
			mkdir ("/EMULATOR/RIN/SAVES", S_IRWXU);
			mkdir ("/EMULATOR/RIN/ROMS", S_IRWXU);
 			RetVal = load_rom (Path);
			if (RetVal > 0) { bMenu = 0; }	
		}
		else if (stricmp (Drivename, /*"sdgeckoa"*/ "fat1") == 0)
		{
			fatMountNormalInterface (PI_SDGECKO_A, 4);
			fatSetDefaultInterface (PI_SDGECKO_A);
			mkdir ("/EMULATOR", S_IRWXU);
			mkdir ("/EMULATOR/RIN", S_IRWXU);
			mkdir ("/EMULATOR/RIN/SAVES", S_IRWXU);
			mkdir ("/EMULATOR/RIN/ROMS", S_IRWXU); 
			RetVal = load_rom (Path);
 			if (RetVal > 0) { bMenu = 0; }	
		}
		else if (stricmp (Drivename, /*"sdgeckob"*/ "fat2") == 0)
		{
			fatMountNormalInterface (PI_SDGECKO_B, 4);
			fatSetDefaultInterface (PI_SDGECKO_B);
			mkdir ("/EMULATOR", S_IRWXU);
			mkdir ("/EMULATOR/RIN", S_IRWXU);
			mkdir ("/EMULATOR/RIN/SAVES", S_IRWXU);
			mkdir ("/EMULATOR/RIN/ROMS", S_IRWXU); 
			RetVal = load_rom (Path); 
			if (RetVal > 0) { bMenu = 0; }	
		}
		else if (stricmp (Drivename, /*"usb"*/ "fat4") == 0)
		{
			fatMountNormalInterface (PI_USBSTORAGE, 4);
			fatSetDefaultInterface (PI_USBSTORAGE);
			mkdir ("/EMULATOR", S_IRWXU);
			mkdir ("/EMULATOR/RIN", S_IRWXU);
			mkdir ("/EMULATOR/RIN/SAVES", S_IRWXU);
			mkdir ("/EMULATOR/RIN/ROMS", S_IRWXU); 
			RetVal = load_rom (Path);
			if (RetVal > 0) { bMenu = 0; }	
		}
		
	}
	
	else if (argc > 0)
	{
		SplitDrivePath (argv[0], &Drivename, &Path);
		
		if (stricmp (Drivename, "frontsd") == 0) 
		{
			fatSetDefaultInterface (PI_INTERNAL_SD);
		}
		else if (stricmp (Drivename, "sdgeckoa") == 0)
		{
			fatMountNormalInterface (PI_SDGECKO_A, 4);
			fatSetDefaultInterface (PI_SDGECKO_A); 
		}
		else if (stricmp (Drivename, "sdgeckob") == 0)
		{
			fatMountNormalInterface (PI_SDGECKO_B, 4);
			fatSetDefaultInterface (PI_SDGECKO_B);
		}
		else if (stricmp (Drivename, "usb") == 0)
		{
			fatMountNormalInterface (PI_USBSTORAGE, 4);
			fatSetDefaultInterface (PI_USBSTORAGE);
		}
	}	
//////////////////////////////////////

	mainloop();
	
	return 0;
}
